# -*- coding: utf-8 -*-
"""
    :Author: woni
    :Date: 2023-01-18 14:42:16
    :Last Modified by:   woni
    :Last Modified time: 2023-01-18 14:42:16
"""

from wedoc.api.base import WedocApiBase


class Form(WedocApiBase):
    def create_form(self):
        pass

    def get_form(self):
        pass

    def get_form_info(self):
        pass

    def get_form_answer(self):
        pass



def _get_column_letter(col_idx):
    """Convert a column number into a column letter (3 -> 'C')

    Right shift the column col_idx by 26 to find column letters in reverse
    order.  These numbers are 1-based, and can be converted to ASCII
    ordinals by adding 64.

    """
    # these indicies corrospond to A -> ZZZ and include all allowed
    # columns
    if not 1 <= col_idx <= 18278:
        raise ValueError("Invalid column index {0}".format(col_idx))
    letters = []
    while col_idx > 0:
        col_idx, remainder = divmod(col_idx, 26)
        # check for exact division and borrow if needed
        if remainder == 0:
            remainder = 26
            col_idx -= 1
        letters.append(chr(remainder + 64))
    return "".join(reversed(letters))


_COL_STRING_CACHE = {}
_STRING_COL_CACHE = {}
for i in range(1, 18279):
    col = _get_column_letter(i)
    _STRING_COL_CACHE[i] = col
    _COL_STRING_CACHE[col] = i


def get_column_letter(
    idx,
):
    """Convert a column index into a column letter
    (3 -> 'C')
    """
    try:
        return _STRING_COL_CACHE[idx]
    except KeyError:
        raise ValueError("Invalid column index {0}".format(idx))


def column_index_from_string(str_col):
    """Convert a column name into a numerical index
    ('A' -> 1)
    """
    # we use a function argument to get indexed name lookup
    try:
        return _COL_STRING_CACHE[str_col.upper()]
    except KeyError:
        raise ValueError("{0} is not a valid column name".format(str_col))



from wedoc.api import Document
from wedoc.api import Spreadsheet
from wedoc.api import Form
from wedoc.client import WedocClientBase


class WedocClient(WedocClientBase):
    doc: Document = Document()
    workbook: Spreadsheet = Spreadsheet()
    form: Form = Form()

    def __init__(self, corpid, corpsecret) -> None:
        """
        对企业微信自建应用初始化

        :param corpid: 企业 ID 标识
        :param corpsecret: 企业自建应用密钥

        使用示例::

        >>> from wedoc import WedocClient
        >>> client = WedocClient("corpid", "corpsecret")
        >>> client.doc.create_doc("doc_name", "doc_type")
        """
        super().__init__(corpid, corpsecret)



import os
import json
import time
import requests
import tempfile
import inspect

from wedoc.api.base import WedocApiBase


def _is_api_endpoint(obj):
    return isinstance(obj, WedocApiBase)


class WedocClientBase:
    def __new__(cls, *args, **kwargs):
        self = super().__new__(cls)
        api_endpoints = inspect.getmembers(self, _is_api_endpoint)
        for name, api in api_endpoints:
            api_cls = type(api)
            api = api_cls(client=self)
            setattr(self, name, api)
        return self

    def __init__(self, corpid, corpsecret) -> None:
        self.base_url = "https://qyapi.weixin.qq.com/cgi-bin"
        self.corpid = corpid
        self.corpsecret = corpsecret
        self.access_token = self.get_local_session()

    def get_local_session(self):
        """创建企业微信应用 access_token 缓存"""
        tmp_path = tempfile.gettempdir()
        wework_app_token_path = os.path.join(tmp_path, "wework_app_token")

        if not os.path.exists(wework_app_token_path):
            return self.set_local_session()

        with open(wework_app_token_path) as f:
            token_dict = json.load(f)
            if int(time.time()) > token_dict.get("expires_in_timestamp"):
                return self.set_local_session()
            else:
                return token_dict.get("access_token")

    def set_local_session(self):
        tmp_path = tempfile.gettempdir()
        wework_app_token_path = os.path.join(tmp_path, "wework_app_token")

        access_token = self.get_access_token().get("access_token")
        token_dict = {"access_token": access_token}
        token_dict["expires_in_timestamp"] = int(time.time()) + 7000
        with open(wework_app_token_path, "w") as f:
            json.dump(token_dict, f)
        return access_token

    def get_access_token(self):
        """获取企业微信应用的 access_token"""
        url = f"{self.base_url}/gettoken?corpid={self.corpid}&corpsecret={self.corpsecret}"
        payload = {}
        headers = {}
        response = requests.request("GET", url, headers=headers, data=payload)
        return response.json()

    def request(self, method, api, params="", pyload={}):
        """发送请求"""
        url = f"{self.base_url}{api}?access_token={self.get_local_session()}"
        headers = {"Content-Type": "application/json"}
        res = requests.request(
            method, url, params=params, data=json.dumps(pyload), headers=headers
        )
        res = res.json()
        if res.get("errcode") != 0:
            raise ValueError(res.get("errmsg"))
        return res


if __name__ == "__main__":
    pass



import json


from wedoc.api.base import WedocApiBase
from wedoc.utils import column_index_from_string, get_column_letter


class Spreadsheet(WedocApiBase):
    def __init__(self, docid: str = None, client: object = None) -> None:
        super().__init__(client)
        self.docid = docid
        self.sheet_metadata = None

    def get_sheet_properties(self) -> dict:
        """获取表格行列信息

        使用示例
        """

        api = "/wedoc/spreadsheet/get_sheet_properties"
        pyload = {"docid": self.docid}
        res = self.request("post", api, pyload=pyload)
        return res.get("properties")

    def get_sheet_property(self, sheet_id: str, property: str):
        """
        通过 sheet_id 找到 sheet 页对应的属性

        :param sheet_id:
        :param property:
        :return:
        """
        for item in self.sheet_metadata:
            if item.get("sheet_id") == sheet_id:
                return item.get(property)
        else:
            raise ValueError("找不到 %s sheet_id 对应的 sheet 页" % property)

    def get_sheet_range_data(self, pyload):
        """
        获取表格行列信息
        :param docid:
        :param sheet_id:
        :param range:
        :return:

        使用示例

        >>> from wedoc import WedocClient
        >>> client = WedocClient("corpid", "corpsecret")
        >>> client.wb.get_sheet_range_data("docid", "range")
        """
        api = "/wedoc/spreadsheet/get_sheet_range_data"
        res = self.request("post", api, pyload=pyload)
        return res

    def batch_update(self, pyload):
        """
        编辑表格内容
        :param docid:
        :return:

        使用示例

        >>> from wedoc import WedocClient
        >>> client = WedocClient("corpid", "corpsecret")
        >>> client.wb.batch_update("docid")
        """

        api = "/wedoc/spreadsheet/batch_update"

        res = self.request("post", api, pyload=pyload)
        return res

    def get_sheet_names(self) -> list:
        """
        获取所有的 sheet 页数据
        :return:
        """
        res = self.get_sheet_properties()
        return [item.get("title") for item in res]

    def get_active_sheet(self) -> str:
        res = self.get_sheet_properties()
        return [item.get("title") for item in res][0]

    def get_sheet_id(self, sheet_name):
        for item in self.sheet_metadata:
            if item.get("title") == sheet_name:
                return item.get("sheet_id")
        else:
            raise ValueError("找不到 s% sheet 页" % sheet_name)

    def add_sheet(
        self, sheet_name: str, row_count: int = 10, column_count: int = 10
    ) -> dict:
        """
        添加 sheet 页
        :param docid: str,
        :param sheet_name: str,
        :param row_count: int = 10,
        :param column_count: int = 10,
        :return:

        """
        pyload = {
            "docid": self.docid,
            "requests": [
                {
                    "add_sheet_request": {
                        "title": sheet_name,
                        "row_count": row_count,
                        "column_count": column_count,
                    }
                },
            ],
        }
        res = self.batch_update(pyload=pyload)
        self.sheet_metadata = self.get_sheet_properties()
        return res

    def delete_sheet(self, sheet_name: str) -> dict:
        """
        删除 sheet 页
        :param sheet_id: str
        :retrun:
        """

        sheet_id = self.get_sheet_id(sheet_name=sheet_name)
        pyload = {
            "docid": self.docid,
            "requests": [
                {"delete_sheet_request": {"sheet_id": sheet_id}},
            ],
        }
        res = self.batch_update(pyload=pyload)
        self.sheet_metadata = self.get_sheet_properties()
        return res

    def rename_sheet(self):
        """
        重命名 sheet 页, 暂不支持
        """
        pass

    def get_row_count(self, sheet_name: str = None):
        """
        获取总行数
        :param sheet_name: str
        :return: int
        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)

        for item in self.sheet_metadata:
            if item.get("sheet_id") == sheet_id:
                return item.get("row_count")

    def get_column_count(self, sheet_name=None):
        """
        获取总列数
        :param sheet_name: str
        :return: int
        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)

        for item in self.sheet_metadata:
            if item.get("sheet_id") == sheet_id:
                return item.get("column_count")

    def get_cell(self, row, column, sheet_name=None):
        """
        读取单元格
        :param row:
        :param column:
        :param sheet_name:
        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)
        pyload = {
            "docid": self.docid,
            "sheet_id": sheet_id,
            "range": f"{column}{row}:{column}{row}",
        }
        res = self.get_sheet_range_data(pyload=pyload)
        res = (
            res.get("grid_data")
            .get("rows")[0]
            .get("values")[0]
            .get("cell_value")
            .get("text")
        )
        return res

    def set_cell(self, row, column, value, sheet_name=None):
        """
        设置单元格数据

        :param row:
        :param column:
        :param value:
        :param sheet_name:
        :return:

        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)
        pyload = {
            "docid": self.docid,
            "requests": [
                {
                    "update_range_request": {
                        "sheet_id": sheet_id,
                        "grid_data": {
                            "start_row": int(row) - 1,
                            "start_column": column_index_from_string(column) - 1,
                            "rows": [{"values": [{"cell_value": {"text": value}}]}],
                        },
                    }
                },
            ],
        }
        res = self.batch_update(pyload)
        return res

    def get_range(self, des_range, sheet_name=None):
        """
        读取区域数据
        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)
        pyload = {"docid": self.docid, "sheet_id": sheet_id, "range": des_range}
        res = self.get_sheet_range_data(pyload=pyload)

        data = []
        for row_data in res.get("grid_data").get("rows"):
            row_tmp_data = []
            for column_data in row_data.get("values"):
                cell_value = column_data.get("cell_value").get("text")
                row_tmp_data.append(cell_value)
            data.append(row_tmp_data)

        return data

    def set_range(self, row, column, data, sheet_name=None):
        """
        设置区域内容
        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)

        range_data = []

        for row_item in data:
            tmp_row = []
            for column_item in row_item:
                tmp_row.append({"cell_value": {"text": column_item}})
            range_data.append({"values": tmp_row})

        pyload = {
            "docid": self.docid,
            "requests": [
                {
                    "update_range_request": {
                        "sheet_id": sheet_id,
                        "grid_data": {
                            "start_row": int(row) - 1,
                            "start_column": column_index_from_string(column) - 1,
                            "rows": range_data,
                        },
                    }
                },
            ],
        }

        res = self.batch_update(pyload)
        return res

    def set_row(self, row, data, start_column, sheet_name=None):
        """
        设置列内容
        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)

        row_data = [{"cell_value": {"text": item}} for item in data]

        pyload = {
            "docid": self.docid,
            "requests": [
                {
                    "update_range_request": {
                        "sheet_id": sheet_id,
                        "grid_data": {
                            "start_row": int(row) - 1,
                            "start_column": column_index_from_string(start_column) - 1,
                            "rows": [{"values": row_data}],
                        },
                    }
                },
            ],
        }

        res = self.batch_update(pyload)
        return res

    def set_column(self, column, data, start_row, sheet_name=None):
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)

        column_data = [{"values": [{"cell_value": {"text": item}}]} for item in data]

        pyload = {
            "docid": self.docid,
            "requests": [
                {
                    "update_range_request": {
                        "sheet_id": sheet_id,
                        "grid_data": {
                            "start_row": int(start_row) - 1,
                            "start_column": column_index_from_string(column) - 1,
                            "rows": column_data,
                        },
                    }
                },
            ],
        }
        res = self.batch_update(pyload)
        return res

    def get_row(self, row, sheet_name=None) -> list:
        """
        读取行内容
        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)
        column_count = self.get_sheet_property(sheet_id, "column_count")

        if column_count == 0:
            return []

        pyload = {
            "docid": self.docid,
            "sheet_id": sheet_id,
            "range": f"A{row}:{get_column_letter(column_count)}{row}",
        }
        res = self.get_sheet_range_data(pyload=pyload)

        data = []
        for row_data in res.get("grid_data").get("rows"):
            row_tmp_data = []
            for column_data in row_data.get("values"):
                cell_value = column_data.get("cell_value").get("text")
                row_tmp_data.append(cell_value)
            data.append(row_tmp_data)

        return data[0]

    def get_column(self, column, sheet_name=None):
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)
        row_count = self.get_sheet_property(sheet_id, "row_count")
        if row_count == 0:
            return []

        pyload = {
            "docid": self.docid,
            "sheet_id": sheet_id,
            "range": f"{column}1:{column}{row_count}",
        }
        res = self.get_sheet_range_data(pyload=pyload)

        res = self.get_sheet_range_data(pyload=pyload)

        data = []
        for row_data in res.get("grid_data").get("rows"):
            row_tmp_data = []
            for column_data in row_data.get("values"):
                cell_value = column_data.get("cell_value").get("text")
                row_tmp_data.append(cell_value)
            data.append(row_tmp_data)

        return [item[0] for item in data]

    def append_row(self, data, sheet_name=None):
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)
        row_count = self.get_row_count(sheet_name)

        row_data = [{"cell_value": {"text": item}} for item in data]

        pyload = {
            "docid": self.docid,
            "requests": [
                {
                    "update_range_request": {
                        "sheet_id": sheet_id,
                        "grid_data": {
                            "start_row": row_count,
                            "start_column": 0,
                            "rows": [{"values": row_data}],
                        },
                    }
                },
            ],
        }
        res = self.batch_update(pyload)
        return res

    def delete_row(self, start_index, end_index, sheet_name=None):
        """
        删除行
        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)
        pyload = {
            "docid": self.docid,
            "requests": [
                {
                    "delete_dimension_request": {
                        "sheet_id": sheet_id,
                        "dimension": "ROW",
                        "start_index": start_index,
                        "end_index": end_index,
                    }
                },
            ],
        }
        res = self.batch_update(pyload)
        return res

    def delete_column(self, start_index, end_index, sheet_name=None):
        """删除列

        Attributes:
            start_index (str): 开始删除的列名. 默认为空.

        """
        sheet_name = sheet_name if sheet_name else self.get_active_sheet()
        sheet_id = self.get_sheet_id(sheet_name)
        pyload = {
            "docid": self.docid,
            "requests": [
                {
                    "delete_dimension_request": {
                        "sheet_id": sheet_id,
                        "dimension": "COLUMN",
                        "start_index": start_index,
                        "end_index": end_index,
                    }
                },
            ],
        }
        res = self.batch_update(pyload)
        return res

    def __call__(self, docid):
        """将表格实例化"""
        self.docid = docid
        self.sheet_metadata = self.get_sheet_properties()
        return self



class WedocApiBase:
    """接口基类"""

    def __init__(self, client=None) -> None:
        self._clent = client

    @property
    def base_url(self):
        return self._clent.access_token

    @property
    def corpid(self):
        return self._clent.corpid

    @property
    def corpsecret(self):
        return self._clent.corpsecret

    @property
    def access_token(self):
        return self._clent.access_token

    def request(self, *args, **kwargs):
        return self._clent.request(*args, **kwargs)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['wedoc', 'wedoc.api']

package_data = \
{'': ['*']}

install_requires = \
['requests>=2.28.2,<3.0.0']

setup_kwargs = {
    'name': 'wedoc',
    'version': '0.1.23',
    'description': '企业微信文档接口, 包括文档的创建, 删除, 以及表格的操作',
    'long_description': '# 企业微信文档接口\n\n> 可以完成简单的在线表格增删改查\n\n[文档](https://wedoc.woni.link)\n\n[PYPI](https://pypi.org/project/wedoc)\n\n\n## 案例\n\n```python\nfrom wedoc import WedocClient\n\n\nif __name__ =="__main__":\n    corpid = "xxxxxxxxxxxxx"\n    corpsecret = "xxxxxxxxxxxxx"\n\n    docid = "xxxxxxxxxxxxxxxxxxxxxx"\n    client = WedocClient(corpid, corpsecret)\n    res = client.access_token\n    print(res)\n    doc_type = 4\n    doc_name = "物料"\n    res = client.doc.create_doc(doc_name, doc_type)\n    print(res)\n    res = client.workbook.get_sheet_properties(docid)\n    print(res)\n\n```\n\n## 文档接口\n\n- 创建文档\n- 重命名文档\n- 删除文档\n- 获取文档基础信息\n\n## 表格接口\n\n- 获取表格行列信息\n- 获取表格数据\n- 编辑表格内容\n- 创建 sheet 页\n- 删除 sheet 页\n- 读取总行数\n- 读取总列数\n- 读取单元格内容, 行, 列, 区域\n- 设置单元格内容, 行, 列, 区域\n\n## 联系我\n\n![](./docs/imgs/Snipaste_2023-02-28_01-34-44.png)\n',
    'author': 'wn',
    'author_email': '320753691@qq.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://wedoc.woni.link/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



from wedoc.api.base import WedocApiBase


class Document(WedocApiBase):
    def create_doc(
        self,
        doc_name: str,
        doc_type: int,
        spaceid: str = "",
        fatherid: str = "",
        admin_users: list = [],
    ) -> dict:
        """
        创建文档
        :param doc_name:
        :param doc_type:
        :param spaceid:
        :param fatherid:
        :param admin_users:
        :return:

        使用示例
        
        >>> from wedoc import WedocClient
        >>> client = WedocClient("corpid", "corpsecret")
        >>> client.doc.create_doc("doc_name", "doc_type")
        """
        api = "/wedoc/create_doc"
        pyload = {
            "doc_type": doc_type,
            "doc_name": doc_name,
            "admin_users": admin_users,
        }

        res = self.request("post", api, pyload=pyload)
        return res

    def rename_doc(self, docid: str, new_name: str) -> dict:
        """
        重命名文档

        :param doc_name:
        :param docid:
        :param new_name:
        :return:

        使用示例

        >>> from wedoc import WedocClient
        >>> client = WedocClient("corpid", "corpsecret")
        >>> client.doc.rename_doc("docid", "new_name")
        """
        api = "/wedoc/rename_doc"
        pyload = {"docid": docid, "new_name": new_name}
        res = self.request("post", api, pyload=pyload)
        return res

    def del_doc(self, docid: str) -> dict:
        """
        删除文档

        :param docid:
        :return:

        使用示例

        >>> from wedoc import WedocClient
        >>> client = WedocClient("corpid", "corpsecret")
        >>> client.doc.del_doc("docid")
        """
        api = "/wedoc/del_doc"
        pyload = {"docid": docid}
        res = self.request("post", api, pyload=pyload)
        return res

    def get_doc_base_info(self, docid: str) -> dict:

        """
        获取文档基础信息
        :param docid:
        :return:

        使用示例

        >>> from wedoc import WedocClient
        >>> client = WedocClient("corpid", "corpsecret")
        >>> client.doc.get_doc_base_info("docid")
        """
        api = "/wedoc/get_doc_base_info"
        pyload = {"docid": docid}
        res = self.request("post", api, pyload=pyload)
        return res
